/**
 * @file
 * Implements our progress bar, replacing Turbo's implementation.
 *
 * @see https://www.drupal.org/project/refreshless/issues/3488464
 *   Issue for adapting this out of Turbo.
 */
((ProgressBar, ProgressBarDelay) => { once(
  'refreshless-turbo-progress-bar', document.documentElement,
).forEach((html) => {

  'use strict';

  const progressBarDelay = new ProgressBarDelay(
    Turbo.session.progressBarDelay, new ProgressBar(),
  );

  html.refreshlessDocumentProgressBar = progressBarDelay;

  // Note that this intentionally listens to the fetch request event rather than
  // turbo:click so that the progress bar also kicks in during back/forward
  // history navigation.
  html.addEventListener('turbo:before-fetch-request', (event) => {

    if (
      // Ignore prefetch requests.
      'X-Sec-Purpose' in event.detail.fetchOptions.headers &&
      event.detail.fetchOptions.headers['X-Sec-Purpose'] === 'prefetch' ||
      // We want to specifically ignore forms as those are handled separately
      // via the turbo:submit-start and turbo:submit-end event handlers.
      event.target.matches('form') === true
    ) {
      return;
    }

    progressBarDelay.showVisitAfterDelay();

    const hideSuccess = () => {

      progressBarDelay.hideVisit();

      html.removeEventListener(
        'turbo:fetch-request-error', hideFailure, {once: true},
      )

    };

    const hideFailure = () => {

      progressBarDelay.hideVisit();

      html.removeEventListener(
        'turbo:load', hideSuccess, {once: true},
      )

    };

    html.addEventListener(
      'turbo:load', hideSuccess, {once: true},
    );

    html.addEventListener(
      'turbo:fetch-request-error', hideFailure, {once: true},
    );

  });

  html.addEventListener('turbo:submit-start', (event) => {

    progressBarDelay.showFormAfterDelay();

    const hideSuccess = () => {

      progressBarDelay.hideForm();

      html.removeEventListener(
        'turbo:fetch-request-error', hideFailure, {once: true},
      )

    };

    const hideFailure = () => {

      progressBarDelay.hideForm();

      html.removeEventListener(
        'turbo:submit-end', hideSuccess, {once: true},
      )

    };

    html.addEventListener(
      'turbo:submit-end', hideSuccess, {once: true},
    );

    html.addEventListener(
      'turbo:fetch-request-error', hideFailure, {once: true},
    );

  });

}); })(
  Drupal.RefreshLess.classes.ProgressBar,
  Drupal.RefreshLess.classes.ProgressBarDelay,
);
