<?php

declare(strict_types=1);

namespace Drupal\Tests\refreshless_turbo\FunctionalJavascript;

use Drupal\FunctionalJavascriptTests\WebDriverTestBase;
use Drupal\Tests\refreshless_turbo\FunctionalJavascript\TurboAssertTrait;
use Drupal\Tests\refreshless_turbo\FunctionalJavascript\TurboHtmlDebugTrait;

/**
 * Navigation tests.
 *
 * @group refreshless
 *
 * @group refreshless_turbo
 */
class NavigationTest extends WebDriverTestBase {

  use TurboAssertTrait;

  use TurboHtmlDebugTrait;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['block', 'refreshless_turbo', 'system'];

  /**
   * Test basic navigation using Turbo Drive.
   */
  public function testBasic(): void {

    // Place the local tasks block so that we have Turbo Drive-enabled links to
    // click.
    $this->drupalPlaceBlock('local_tasks_block', [
      'region' => 'content', 'id' => 'local-tasks-block',
    ]);

    // Do a full page load request to start.
    $this->drupalGet('user/login');

    $this->assertSession()->assertTurboIsPresent();

    $this->assertSession()->startTurboPersist();

    // Click a Turbo Drive-enabled link.
    $this->click('[data-drupal-link-system-path="user/register"]');

    $this->assertSession()->assertWaitOnTurboRequest();

    // Assert that the user register form is now available.
    $this->assertSession()->elementExists(
      'css', 'form[data-drupal-selector="user-register-form"]',
    );

    $this->assertSession()->assertTurboPersisted();

    $this->turboHtmlOutput();

    // Click a Turbo Drive-enabled link.
    $this->click('[data-drupal-link-system-path="user/password"]');

    $this->assertSession()->assertWaitOnTurboRequest();

    // Assert that the reset password form is now available.
    $this->assertSession()->elementExists(
      'css', 'form[data-drupal-selector="user-pass"]',
    );

    $this->assertSession()->assertTurboPersisted();

    $this->turboHtmlOutput();

  }

}
